'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _deepmerge = _interopRequireDefault(require("deepmerge"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _firstChild = function _firstChild(node) {
  return node.children[0];
};

var _identity = function _identity(node) {
  return node.value;
};

function _extractComparison(node) {
  var symbol = null;
  var value = null;
  node.children.forEach(function (child) {
    if (child.type === 'SYMBOL') {
      if (symbol) {
        throw new Error('MONGODB: You can only specify one symbol in a comparison.');
      }

      symbol = child;
    } else {
      if (value) {
        throw new Error('MONGODB: You can only specify one value in a comparison.');
      }

      value = child;
    }
  });

  if (!(symbol && value)) {
    throw new Error('MONGODB: Invalid comparison, could not find both symbol and value.');
  }

  return {
    symbol: symbol,
    value: value
  };
}

var MongoCompiler = /*#__PURE__*/function () {
  function MongoCompiler(options) {
    _classCallCheck(this, MongoCompiler);

    _defineProperty(this, "NUMBER", _identity);

    _defineProperty(this, "BOOLEAN", _identity);

    _defineProperty(this, "PRIMITIVE", _identity);

    _defineProperty(this, "STRING", _identity);

    this.flavor = (options === null || options === void 0 ? void 0 : options.flavor) || 'mongodb';
  }

  _createClass(MongoCompiler, [{
    key: "SYMBOL",
    value: function SYMBOL(node) {
      if (node.alone) {
        return _defineProperty({}, node.value, {
          $exists: true
        });
      }

      return node.value;
    }
  }, {
    key: '-',
    value: function _(node) {
      return -_firstChild(node).value;
    }
  }, {
    key: '&&',
    value: function _(node) {
      var _this = this;

      var _and = {
        $and: []
      };
      node.children.forEach(function (_node) {
        _and.$and.push(_this.processNode(_node));
      });
      return _and;
    }
  }, {
    key: '||',
    value: function _(node) {
      var _this2 = this;

      var _or = {
        $or: []
      };
      node.children.forEach(function (_node) {
        _or.$or.push(_this2.processNode(_node));
      });
      return _or;
    }
  }, {
    key: "ARRAY",
    value: function ARRAY(node) {
      return node.children.map(this.processNode.bind(this));
    }
  }, {
    key: "IN",
    value: function IN(node) {
      var field = this.processNode(node.children[0]);
      var valueList = this.processNode(node.children[1]);

      var _in = _defineProperty({}, field, {
        $in: []
      });

      valueList.forEach(function (value) {
        _in[field].$in.push(value);
      });
      return _in;
    }
  }, {
    key: '!',
    value: function _(node) {
      var expr = this.processNode(node.children[0]);
      return {
        $nor: [expr]
      };
    }
  }, {
    key: '==',
    value: function _(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);
      return _defineProperty({}, symbol, value);
    }
  }, {
    key: '!=',
    value: function _(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);
      return _defineProperty({}, symbol, {
        $ne: value
      });
    }
  }, {
    key: "MATCH",
    value: function MATCH(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);

      var _match = _defineProperty({}, symbol, {
        regex: value
      });

      return _match;
    }
  }, {
    key: '<',
    value: function _(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);

      var _lt = _defineProperty({}, symbol, {
        $lt: value
      });

      return _lt;
    }
  }, {
    key: '<=',
    value: function _(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);

      var _lte = _defineProperty({}, symbol, {
        $lte: value
      });

      return _lte;
    }
  }, {
    key: '>',
    value: function _(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);

      var _gt = _defineProperty({}, symbol, {
        $gt: value
      });

      return _gt;
    }
  }, {
    key: '>=',
    value: function _(node) {
      var comparison = _extractComparison(node);

      var symbol = this.processNode(comparison.symbol);
      var value = this.processNode(comparison.value);

      var _gte = _defineProperty({}, symbol, {
        $gte: value
      });

      return _gte;
    }
  }, {
    key: "EXPRESSION",
    value: function EXPRESSION(node) {
      var _this3 = this;

      var expression = {};
      node.children.forEach(function (_node) {
        expression = (0, _deepmerge["default"])(expression, _this3.processNode(_node));
      });
      return expression;
    }
  }, {
    key: "processNode",
    value: function processNode(node) {
      if (!this[node.type]) {
        throw new Error("Invalid node type ".concat(node.type));
      }

      return this[node.type](node);
    }
  }]);

  return MongoCompiler;
}();

var compile = function compile(tree) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var compiler = new MongoCompiler(options);
  return compiler.processNode(tree);
};

var _default = compile;
exports["default"] = _default;
//# sourceMappingURL=index.js.map